/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.controller;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.nacos.common.utils.Md5Utils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.SysDept;
import com.qunzhixinxi.hnqz.admin.api.entity.SysRole;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUser;
import com.qunzhixinxi.hnqz.admin.api.entity.SysUserRole;
import com.qunzhixinxi.hnqz.admin.api.vo.ErrorInfoVo;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.config.UpmsConfig;
import com.qunzhixinxi.hnqz.admin.entity.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.mapper.*;
import com.qunzhixinxi.hnqz.admin.service.SysDeptService;
import com.qunzhixinxi.hnqz.admin.service.SysUserService;
import com.qunzhixinxi.hnqz.admin.service.WmDaAgentService;
import com.qunzhixinxi.hnqz.admin.service.WmDaDrugEntService;
import com.qunzhixinxi.hnqz.admin.util.ValidateUtils;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.ding.service.DingService;
import com.qunzhixinxi.hnqz.common.log.annotation.SysLog;
import com.qunzhixinxi.hnqz.common.qcc.entity.EmployeeGetListDataItem;
import com.qunzhixinxi.hnqz.common.qcc.entity.EmployeeGetListResponse;
import com.qunzhixinxi.hnqz.common.qcc.entity.PageInfo;
import com.qunzhixinxi.hnqz.common.qcc.service.QccService;
import com.qunzhixinxi.hnqz.common.security.annotation.Inner;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.common.taxhelper.enums.TaxHelperCertStatus;
import com.qunzhixinxi.hnqz.common.taxhelper.service.TaxHelperService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @author hnqz
 * @date 2018/12/16
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/user")
@Api(value = "user", tags = "用户管理模块")
public class SysUserController {

	private final SysUserService userService;
	private final SysDeptService sysDeptService;
	private final SysDeptMapper sysDeptMapper;
	private final SysUserMapper sysUserMapper;
	private final RedisTemplate redisTemplate;
	private final DingService dingService;
	private final UpmsConfig upmsConfig;
	private final SysUserRoleMapper sysUserRoleMapper;
	private final SysRoleMapper sysRoleMapper;
	private final TaxHelperService taxHelperService;
	private final WmDaAgentService wmDaAgentService;
	private final WmDaDrugEntService wmDaDrugEntService;
	private final SysCertResultRecordMapper sysCertResultRecordMapper;
	private final QccService qccService;


	private boolean isZbOrDbUsers(Integer[] roles) {
		if (null == roles) {
			return false;
		}

		if (Arrays.asList(roles).contains(1) || Arrays.asList(roles).contains(2) || Arrays.asList(roles).contains(3)
				|| Arrays.asList(roles).contains(4) || Arrays.asList(roles).contains(7)
				|| Arrays.asList(roles).contains(9) || Arrays.asList(roles).contains(11)) {
			// 1-管理员，2-平台管理员，3-药企管理员，4-CSO管理员，7-财务管理员，9-CRO管理员，11-患教管理员
			return true;
		}
		return false;
	}

	/**
	 * 用户管理-认证
	 *
	 * @param input
	 * @return R
	 */
	@ApiOperation(value = "用户管理-认证", notes = "用户管理-认证")
	@SysLog("用户管理-认证")
	@GetMapping("/getAuthInfo")
	public R getAuthInfo(UserCertificationInput input) {
		//参数校验
		return userService.getAuthInfo(input);
	}

	/**
	 * 用户管理-认证-确定
	 *
	 * @param input
	 * @return R
	 */
	@ApiOperation(value = "用户管理-认证-确定", notes = "用户管理-认证-确定")
	@SysLog("用户管理-认证-确定")
	@PostMapping("/batchChannelCert")
	public R batchChannelCert(@RequestBody UserCertificationInput input) {
		//参数校验
		return userService.batchChannelCert(input);
	}


	/**
	 * 获取指定用户全部信息
	 *
	 * @return 用户信息
	 */
	@Inner
	@GetMapping("/info/{username}")
	public R info(@PathVariable String username) {

		// //排除众包用户
		// List<Integer> roleList=new ArrayList<>();
		// roleList.add(5); //医药代表
		// roleList.add(6); //众包代表
		// QueryWrapper<SysUserRole> wrappers=Wrappers.query();
		//
		// List<SysUserRole>
		// list=sysUserRoleService.list(wrappers.lambda().notIn(SysUserRole::getRoleId,roleList));
		// List<Integer> listZbUserList=new ArrayList<>();
		// for (SysUserRole sysUserRole:list) {
		// listZbUserList.add(sysUserRole.getUserId());
		// }
		//
		// if(null==listZbUserList)
		// {
		// return R.failed(null, String.format("用户信息为空 %s", username));
		// }

		// 根据用户名查询用户时，过滤掉已删除和已禁用的用户
		List<SysUser> user = userService.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username)
				.eq(SysUser::getDelFlag, 0).eq(SysUser::getLockFlag, 0));

		if (user == null) {
			return R.failed(null, String.format("用户信息为空 %s", username));
		}

		if (user.size() == 1) {
			UserInfo userInfo = userService.findUserInfo(user.get(0));
			// 校验用户角色是否包含众包用户和医药代表用户

			if (isZbOrDbUsers(userInfo.getRoles())) {
				return R.ok(userInfo);
			} else {
				return R.failed(null, String.format("用户信息为空 %s", username));
			}
		} else {
			String password = redisTemplate.opsForValue().get(username) + "";
			if (null != password) {
				for (int i = 0; i < user.size(); i++) {
					UserInfo userInfo = userService.findUserInfo(user.get(i));
					if (Md5Utils.getMD5(password.getBytes()).equals(user.get(i).getW1())) {
						if (isZbOrDbUsers(userInfo.getRoles())) {
							redisTemplate.delete(username);
							return R.ok(userInfo);
						}
					}
				}
			}
		}
		return R.failed(null, String.format("用户信息为空 %s", username));
	}

	/**
	 * 通过ID查询用户信息
	 *
	 * @param id ID
	 * @return 用户信息
	 */
	@GetMapping("/{id}")
	public R user(@PathVariable Integer id) {
		return R.ok(userService.selectUserVoById(id));
	}

	/**
	 * 根据用户名查询用户信息
	 *
	 * @param username 用户名
	 * @return
	 */
	@GetMapping("/details/{username}")
	public R user(@PathVariable String username) {
		SysUser condition = new SysUser();
		condition.setUsername(username);
		return R.ok(userService.getOne(new QueryWrapper<>(condition)));
	}

	/**
	 * 根据用户名查询用户信息
	 *
	 * @param username 用户名
	 * @return
	 */
	@GetMapping("/getDetails")
	public R getuser(String deptId, String username) {
		SysUser condition = new SysUser();
		condition.setUsername(username);
		SysDept sysDept1 = sysDeptService.getById(Integer.parseInt(deptId));
		if (null != sysDept1) {
			condition.setDeptId(sysDept1.getDeptId());
		}
		return R.ok(userService.getOne(new QueryWrapper<>(condition)));
	}

	/**
	 * 删除用户信息
	 *
	 * @param id ID
	 * @return R
	 */
	@SysLog("删除用户信息")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('sys_user_del')")
	@ApiOperation(value = "删除用户", notes = "根据ID删除用户")
	@ApiImplicitParam(name = "id", value = "用户ID", required = true, dataType = "int", paramType = "path")
	public R userDel(@PathVariable Integer id) {
		SysUser sysUser = userService.getById(id);
		return R.ok(userService.deleteUserById(sysUser));
	}

	/**
	 * 添加用户
	 *
	 * @param userPwd 用户信息
	 * @return success/false
	 */
	@Inner(value = false)
	@SysLog("修改密码")
	@PostMapping("/upd")
	public R updatePwd(@RequestBody @Valid UserPwd userPwd) {

		return userService.updatePwd(userPwd);
	}

	/**
	 * 添加用户
	 *
	 * @param userDto 用户信息
	 * @return success/false
	 */
	@SysLog("添加用户")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_user_add')")
	public R user(@RequestBody UserDTO userDto) {
		// if (SecurityUtils.getRoles().contains(2) ||
		// SecurityUtils.getRoles().contains(1)) {
		SysDept sysDept = sysDeptService.getById(userDto.getDeptId());

		// TODO qcc调用企查查查询是否为主要人员
		/**
		 * @Modify: start qcc调用企查查查询是否为主要人员
		 * @Version: v2021.4.16
		 * @Author: ryz
		 * @Date: 2021/5/6
		*/
		if (userDto.getRole().contains(6) || userDto.getRole().contains(10)) {
			Map<String, String> params = new HashMap<>();
			params.put("searchKey", sysDept.getName());
			params.put("pageIndex", "1");
			params.put("pageSize", "10");
			EmployeeGetListResponse qccRes = qccService.getList(params);
			if (qccRes.isSuccess()) {
				for (EmployeeGetListDataItem item : qccRes.getResult()) {
					if (userDto.getRealname().equals(item.getName())) {
						if(userDto.getRole().contains(10)){
							return R.failed(String.format("%s为该公司%s，不能为兼职CRC",item.getName(),item.getJob()));
						}else{
							return R.failed(String.format("%s为该公司%s，不能为兼职学术推广员",item.getName(),item.getJob()));
						}

					}
				}
				if(null != qccRes.getPaging()){
					PageInfo qccPage = qccRes.getPaging();
					// 需要分页查询
					if (qccPage.getTotalRecords() > qccPage.getPageIndex() * qccPage.getPageSize()) {
						int totalPage = qccPage.getTotalRecords() / qccPage.getPageSize();

						for (int i = 2; i < totalPage + 2; i++) {
							params.put("pageIndex", i + "");
							qccRes = qccService.getList(params);
							if (qccRes.isSuccess()) {
								for (EmployeeGetListDataItem item : qccRes.getResult()) {
									if (userDto.getRealname().equals(item.getName())) {
										if(userDto.getRole().contains(10)){
											return R.failed(String.format("%s为该公司%s，不能为兼职CRC",item.getName(),item.getJob()));
										}else{
											return R.failed(String.format("%s为该公司%s，不能为兼职学术推广员",item.getName(),item.getJob()));
										}
									}
								}
							} else {
								break;
							}
						}
					}
				}

			}
		}

		// 根据结算主体类型设置用户类型
//		if ("1".equals(sysDept.getSubjectType())) {
//			userDto.setUserType("1");
//			userDto.setCertStatus(TaxHelperCertStatus.UN_CERT.getCode());
//		} else {
//			userDto.setUserType("0");
//		}
		if(ArrayUtils.contains(sysDept.getSubjectType(),SubjectTypeEnum.TYPE_SHUIBANGYUN.getCode())){
			userDto.setCertStatus(TaxHelperCertStatus.UN_CERT.getCode());
		}
//		if(ArrayUtils.contains(sysDept.getSubjectType(),SubjectTypeEnum.TYPE_RENLIJIA.getCode())){
//			userDto.setRljCertStatus(DingEnum.CER_STATUS_4.getType());
//		}


		if (userDto.getRole().contains(2)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 3) {
								return R.failed("用户已存在平台管理员角色,不能同时为药企管理员");
							}
							if (sysRole.getRoleId() == 4) {
								return R.failed("用户已存在平台管理员角色,不能同时为CSO管理员");
							}
							if (sysRole.getRoleId() == 5) {
								return R.failed("用户已存在平台管理员角色,不能同时为全职学术推广员");
							}
							if (sysRole.getRoleId() == 6) {
								return R.failed("用户已存在平台管理员角色,不能同时为兼职学术推广员");
							}
							if (sysRole.getRoleId() == 7) {
								return R.failed("用户已存在平台管理员角色,不能同时为财务管理员");
							}
						}
					}
				}
			}

		} else if (userDto.getRole().contains(3)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在药企管理员角色,不能同时为平台管理员");
							}

							if (sysRole.getRoleId() == 4) {
								return R.failed("用户已存在药企管理员角色,不能同时为CSO管理员");
							}
//							SysDept sysDept = sysDeptService.getById(userVO.getDeptId());
							if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
								if (sysRole.getRoleId() == 5) {
									return R.failed("用户已存在CSO关系角色,不能同时为药企管理员");
								}
							}
						}
					}
				}
			}

		} else if (userDto.getRole().contains(4)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在平台管理员角色,不能同时为CSO管理员");
							}
							if (sysRole.getRoleId() == 4) {
								return R.failed("用户已存在CSO管理员角色,不能为其他机构CSO管理员");
							}
							if (sysRole.getRoleId() == 9) {
								return R.failed("用户已存在CRO管理员,不能同时为CSO管理员");
							}

							if (sysRole.getRoleId() == 3) {
								return R.failed("用户已存在药企管理员角色,不能同时为CSO管理员");
							}
							if (sysRole.getRoleId() == 5) {
								SysUserRole sysUserRole = new SysUserRole();
								sysUserRole.setUserId(userVO.getUserId());
								List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
								for (int i = 0; i < sysUserRoleList.size(); i++) {
									SysUser sysUser = userService.getById(sysUserRoleList.get(i).getUserId());
									if (!sysUser.getDeptId().equals(userDto.getDeptId())) {
										return R.failed("用户已存在全职学术推广员角色,不能为其他机构CSO管理员");
									}
								}
							}
						}
					}
				}
			}

		} else if (userDto.getRole().contains(7)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在平台管理员,不能同时为财务管理员");
							}
//							SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
							if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
								if (sysRole.getRoleId() == 3) {
									return R.failed("用户已存在药企管理员,不能同时为CSO机构财务管理员");
								}
							}
						}
					}
				}
			}

		} else if (userDto.getRole().contains(5)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在平台管理员,不能同时为全职学术推广员");
							}
							if (sysRole.getRoleId() == 3) {
//								SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
								if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
									return R.failed("用户已存在药企管理员,不能为CSO的全职学术推广员");
								}
							}
							if (sysRole.getRoleId() == 4) {
//								SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
								if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
									return R.failed("用户已存在CSO管理员,不能为其他CSO的全职学术推广员");
								}
							}

						}
					}
				}
			}

		} else if (userDto.getRole().contains(6)) {

			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在平台管理员,不能同时为兼职学术推广员");
							}
							SysUserRole sysUserRole = new SysUserRole();
							sysUserRole.setUserId(userVO.getUserId());
							List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
							for (int i = 0; i < sysUserRoleList.size(); i++) {
								if (sysUserRoleList.get(i).getRoleId().equals(7)) {
									SysUser sysUser = userService.getById(sysUserRoleList.get(i).getUserId());
									if (sysUser.getDeptId().equals(userDto.getDeptId())) {
										return R.failed("用户已存在财务管理员角色，不能同时为兼职学术推广员");
									}
								}
							}
						}
					}
				}
			}

		} else if (userDto.getRole().contains(9)) {
			UserDTO userDTO = new UserDTO();
			userDTO.setUsername(userDto.getUsername());
			userDTO.setDelFlag("0");
			List<UserVO> userDTOList = sysUserMapper.selectByPhone(userDTO);
			if (userDTOList.size() > 0) {
				for (UserVO userVO : userDTOList) {
					if (null != userVO.getRoleList()) {
						for (SysRole sysRole : userVO.getRoleList()) {
							if (sysRole.getRoleId() == 2) {
								return R.failed("用户已存在平台管理员,不能同时为CRO管理员");
							}
							if (sysRole.getRoleId() == 3) {
								return R.failed("用户已存在药企管理员,不能同时为CRO管理员");
							}
							if (sysRole.getRoleId() == 4) {
								return R.failed("用户已存在CSO管理员角色,不能同时为CRO管理员");
							}
							if (sysRole.getRoleId() == 9) {
								return R.failed("用户已存在CRO管理员角色");
							}
						}
					}
				}
			}

		}
		if (userDto.getRole().contains(3)) {
//			SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
			if (sysDept.getLevel() != 2) {
				return R.failed("角色选为药企管理员时,组织架构必须为药企");
			}
		}
		// }
		if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
			if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
				return R.failed("不能跨机构维护学术推广员");
			}
		}
		SysUser query = new SysUser();
		query.setUsername(userDto.getUsername());
		query.setDeptId(userDto.getDeptId());
//		SysUser sysUser = sysUserMapper.selectUserByName(query);
		Integer count = sysUserMapper.selectCount(Wrappers.query(query));
		// TODO  用户是否存在
		if (count != 0) {
			return R.failed("当前机构已存在该账号");
		}

//		if (sysUser != null) {
//			SysUserRole sysUserRole = new SysUserRole();
//			sysUserRole.setUserId(sysUser.getUserId());
//			List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
//			List<Integer> list = sysUserRoleList.stream().map(p -> p.getRoleId()).collect(Collectors.toList());
//			for (Integer id : list) {
//				if (userDto.getRole().contains(3)) {
////					SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());
//					if (sysDept.getLevel() != 2) {
//						return R.failed("此账号已存在管理员角色");
//					}
//					if (id.equals(2)) {
//						return R.failed("用户已存在平台管理员管理员角色,不能同时为CSO管理员角色");
//					}
//					if (id.equals(3)) {
//						return R.failed("用户存在药企管理员角色");
//					}
//				}
//				if (userDto.getRole().contains(4)) {
//					if (id.equals(3) || id.equals(4)) {
//						return R.failed("用户已存在管理员角色");
//					}
//					if (id.equals(2)) {
//						return R.failed("用户已存在平台管理员管理员角色,不能同时为CSO管理员角色");
//					}
//				}
//				if (userDto.getRole().contains(5)) {
//					if (id.equals(3)) {
////						SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
//						if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
//							return R.failed("用户存在药企管理员角色,不能维护为经销商组织的全职学术推广员角色");
//						}
//					}
//					if (id.equals(5)) {
//						return R.failed("用户已存在全职学术推广员角色");
//					}
//					if (id.equals(2)) {
//						return R.failed("用户已存在平台管理员管理员角色,不能同时为全职学术推广员角色");
//					}
//					if (id.equals(6)) {
//						return R.failed("用户已存在兼职学术推广员角色,不能同时为全职学术推广员角色");
//					}
//					if (id.equals(4)) {
//						if (!userDto.getDeptId().equals(sysUser.getDeptId())) {
//							return R.failed("用户已存在CSO管理员角色,不能维护为其他经销商组织的全职学术推广员角色");
//						}
//					}
//
//				}
//				if (userDto.getRole().contains(6)) {
//					if (id.equals(3)) {
//						return R.failed("用户已存在药企管理员角色,不能同时为兼职学术推广员角色");
//					}
//					if (id.equals(6)) {
//						return R.failed("用户已存在兼职学术推广员角色");
//					}
//					if (id.equals(2)) {
//						return R.failed("用户已存在平台管理员管理员角色,不能同时为兼职学术推广员角色");
//					}
//					if (id.equals(4)) {
//						return R.failed("用户不能同时为CSO管理员角色和兼职学术推广员角色");
//					}
//					if (id.equals(5)) {
//						return R.failed("用户已存在全职学术推广员角色,用户不能同时为兼职学术推广员角色");
//					}
//					if (id.equals(7)) {
//						if (!userDto.getDeptId().equals(sysUser.getDeptId())) {
//							return R.failed("用户已存在财务管理员角色,用户不能同时为兼职学术推广员角色");
//						}
//					}
//				}
//				if (userDto.getRole().contains(7)) {
//					if (id.equals(6)) {
//						return R.failed("用户不能同时为财务管理员角色和兼职学术推广员角色");
//					}
//					if (id.equals(4)) {
//						return R.failed("用户存在本机构CSO管理员角色，不能为兼职学术推广员角色");
//					}
//					if (id.equals(3)) {
//						return R.failed("用户存在本机构药企管理员角色，不能为兼职学术推广员角色");
//					}
//					if (id.equals(2)) {
//						return R.failed("用户已存在平台管理员管理员角色,不能同时为兼职学术推广员角色");
//					}
//				}
//				if (userDto.getRole().contains(9)) {
//					if (id.equals(9)) {
//						return R.failed("用户在本机构已存在CRO管理员角色");
//					}
//				}
//				if (userDto.getRole().contains(10)) {
//					if (id.equals(10)) {
//						return R.failed("用户在本机构已存在兼职CRC角色");
//					}
//				}
//			}
//			List<Integer> list2 = userDto.getRole();
//			for (Integer id : list2) {
//				sysUserRole.setRoleId(id);
//				sysUserRoleMapper.insert(sysUserRole);
//			}
//			return R.ok();
//		}

		/**
		 * @Modify: start
		 * @Version: v2021.4.16
		 * @Author: ryz
		 * @Date: 2021/4/29
		 * 去掉认证
		 */
//		if (null != userDto.getDeptId() && StringUtils.isNotEmpty(userDto.getIdCardNumber())) {
////			SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
//			if (null != sysDept && StringUtils.isNotEmpty(sysDept.getAppId())) {
//				if ("1".equals(userDto.getUserType())) {
//					TaxHelperConfig config = new TaxHelperConfig();
//					config.setAppId(sysDept.getAppId());
//					config.setAppSecrete(sysDept.getAppSecret());
//					config.setQueryUrl(sysDept.getQueryUrl());
//
//					TaxHelperAddMemberModel model = new TaxHelperAddMemberModel(config);
//					model.setIdCard(userDto.getIdCardNumber());
//					model.setPhone(userDto.getUsername());
//					model.setName(userDto.getRealname());
//					model.setNotifyUrl("http://localhost:8808");
//					String entCode = null;
////					switch (sysDept.getLevel()) {
////						case 1:
////							entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel1Id(), 1);
////							break;
////						case 2:
////							entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel2Id(), 2);
////							break;
////						case 3:
////							entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel3Id(), 3);
////							break;
////					}
////					model.setEnterpriseCode(entCode);
//
//					WmDaAgent agent = wmDaAgentService.selectByDeptId(sysDept.getDeptId());
//					WmDaDrugEnt drugEnt = wmDaDrugEntService.selectByDeptId(sysDept.getDeptId());
//					if (null != agent) {
//						model.setEnterpriseCode(agent.getZjNumber());
//					} else {
//						model.setEnterpriseCode(drugEnt.getEntcode());
//					}
//					if (StringUtils.isEmpty(model.getEnterpriseCode())) {
//						return R.failed("未配置企业的统一社会信用代码");
//					}
//
//					TaxHelperAddMemberRequest request = new TaxHelperAddMemberRequest(model);
//					TaxHelperResponse response = taxHelperService.addMembers(request);
//
//					// 记录认证结果
//					SysCertResultRecord record = new SysCertResultRecord();
//					record.setCode(response.getCode());
//					record.setCreateTime(LocalDateTime.now());
//					record.setDeptId(sysDept.getDeptId());
//					record.setMessage(StringUtils.isNotEmpty(response.getMessage())?response.getMessage():"无");
//					record.setStatus(null);
//					record.setSubjectType(1);
//					record.setMethod("增员");
//					record.setUserId(sysUser.getUserId());
//					sysCertResultRecordMapper.insert(record);
//
//					if (response.success()) {
//						userDto.setCertStatus(TaxHelperCertStatus.ADD_MEMBER.getCode());
//					} else {
//						log.info("税邦云增员失败：{},{}，原因是：{}", userDto.getRealname(), userDto.getUsername(),
//								response.getMessage());
//						userDto.setIdCardNumber(null);
//					}
//
//				} else {
//					DingConfig config = new DingConfig();
//					config.setAppId(sysDept.getAppId());
//					config.setAppSecret(sysDept.getAppSecret());
//					config.setQueryUrl(sysDept.getQueryUrl());
//
//					Map map = new HashMap();
//					map.put("name", userDto.getRealname());
//					map.put("mobile", userDto.getUsername());
//					map.put("idCard", userDto.getIdCardNumber());
//					map.put("idType", "1");
//					List rzlist = new ArrayList();
//					rzlist.add(map);
//
//					EmployCheckResponse response = dingService.check(config, rzlist);
//					if (!response.isSuccess()) {
//						return R.failed("人力家接口调用失败");
//					}
//					// 接口请求成功，但是接口返回的状态还需区分checkStatus，1:验证通过，状态改成已提交，2:验证不通过，状态改成认证失败
//					List<EmployCheckResultItem> freelancesResult = response.getData().getFreelancesResult();
//					if (CollectionUtils.isNotEmpty(freelancesResult)) {
//						for (EmployCheckResultItem employCheckResultItem : freelancesResult) {
//							if (employCheckResultItem.getIdCard().equals(userDto.getIdCardNumber())) {
//								if (employCheckResultItem.getCheckStatus() == 1) {// 验证通过
//									userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//									continue;
//								} else {// 验证不通过
//									if ("1014".equals(employCheckResultItem.getCheckRemark())) {
//										log.info("认证返回1014按成功处理：{},{}", userDto.getRealname(), userDto.getUsername());
//										userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//										continue;
//									}
//									if ("1018".equals(employCheckResultItem.getCheckRemark())) {
//										log.info("认证返回1018按处理中处理：{},{}", userDto.getRealname(), userDto.getUsername());
//										userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//										continue;
//									}
//									if ("1019".equals(employCheckResultItem.getCheckRemark())) {
//										log.info("认证返回1019按成功处理：{},{}", userDto.getRealname(), userDto.getUsername());
//										userDto.setCertStatus(DingEnum.CER_STATUS_1.getType());
//										continue;
//									}
//									if ("1020".equals(employCheckResultItem.getCheckRemark())) {
//										log.info("认证返回1020按认证失败处理：{},{}", userDto.getRealname(), userDto.getUsername());
//										userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//										continue;
//									}
//									userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//									log.info("认证失败:" + employCheckResultItem.getCheckRemark());
//								}
//							}
//						}
//					} else {
//						userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//					}
//				}
//			}
//		}
		//TODO 身份证年龄
		if (userDto.getIdCardNumber().length() > 0) {
			String str = userDto.getIdCardNumber();
			str = str.substring(6, 10);
			String str1 = userDto.getIdCardNumber();
			str1 = str1.substring(10, 12);
			String str2 = userDto.getIdCardNumber();
			str2 = str2.substring(12, 14);
			int i = Integer.parseInt(str);
			int i1 = Integer.parseInt(str1) - 1;
			int i2 = Integer.parseInt(str2);

			Calendar calendar = Calendar.getInstance();
			calendar.setTime(new Date());//放入Date类型数据

			int year = calendar.get(Calendar.YEAR);//获取年份
			int month = calendar.get(Calendar.MONTH);//获取月份
			int day = calendar.get(Calendar.DATE);//获取日

			if (year - i < 18) {
				return R.failed("年龄未满十八");
			} else if (year - i == 18) {
				if (month > i1) {
					return R.failed("年龄未满十八");
				} else if (month == i1) {
					if (day < i2) {
						return R.failed("年龄未满十八");
					}
				}
			} else if (year - i > 65) {
				return R.failed("年龄已超过六十五");
			} else if (year - i == 65) {
				if (month > i1) {
					return R.failed("年龄已超过六十五");
				} else if (month == i1) {
					if (day >= i2) {
						return R.failed("年龄已超过六十五");
					}
				}
			}
		}

		return R.ok(userService.saveUser(userDto));
	}


	/**
	 * 批量导入用户
	 *
	 * @param file 用户信息文件
	 * @return success/false
	 */
	@SysLog("批量导入用户")
	@PostMapping(value = "/batch")
	@PreAuthorize("@pms.hasPermission('sys_user_add')")
	public R batchUser(@RequestPart("file") MultipartFile file) {

		// 获取操作员和操作员所在的结算主体
		HnqzUser operator = SecurityUtils.getUser();
		SysDept sysDept = sysDeptService.getById(operator.getDeptId());

		// 读取excel
		try {
			String fileName = file.getOriginalFilename().toLowerCase();
			InputStream inputStream = file.getInputStream();

			// 校验文件信息
			Workbook wbs = null;
			if (fileName.endsWith("xls")) {
				//2003
				wbs = new HSSFWorkbook(inputStream);
			} else if (fileName.endsWith("xlsx")) {
				//2007
				wbs = new XSSFWorkbook(inputStream);
			} else {
				return R.failed("上传文件格式错误!");
			}
			if (wbs.isSheetHidden(0)) {
				return R.failed("导入表中有隐藏列");
			}

			// 校验页（数据页）
			Sheet sheet = wbs.getSheetAt(1);

			// 校验模板
			Row row0 = sheet.getRow(0);
			if (row0 == null) {
				return R.failed("导入模版错误！");
			}

			// 校验记录数
			int rowNum = sheet.getLastRowNum();
			if (rowNum < 1) {
				return R.failed("导入表中缺少数据！");
			} else if (rowNum > 1000) {
				return R.failed("单次批量导入最大支持1000行数据！");
			}

			// 校验列信息
			String name = row0.getCell(0).getStringCellValue();
			if (!"姓名".equals(name)) {
				return R.failed("导入模版第一列错误！");
			}
			String phone = row0.getCell(1).getStringCellValue();
			if (!"手机号".equals(phone)) {
				return R.failed("导入模版第二列错误！");
			}
			String prov = row0.getCell(2).getStringCellValue();
			if (!"省".equals(prov)) {
				return R.failed("导入模版第三列错误！");
			}
			String city = row0.getCell(3).getStringCellValue();
			if (!"市".equals(city)) {
				return R.failed("导入模版第四列错误！");
			}
			String role = row0.getCell(4).getStringCellValue();
			if (!"角色".equals(role)) {
				return R.failed("导入模版第五列错误！");
			}
			String userStatus = row0.getCell(5).getStringCellValue();
			if (!"状态".equals(userStatus)) {
				return R.failed("导入模版第六列错误！");
			}
			String provinceCode = row0.getCell(6).getStringCellValue();
			if (!"省份编码".equals(provinceCode)) {
				return R.failed("导入模版第七列错误！");
			}
			String cityCode = row0.getCell(7).getStringCellValue();
			if (!"城市编码".equals(cityCode)) {
				return R.failed("导入模版第八列错误！");
			}

			//数据
			List<UserDTO> userDTOList = new ArrayList<>();
			List<ErrorInfoVo> errorList = new ArrayList<>();
			log.info(">>> 通过excel批量创建用户开始 <<<");
			// 行数据
			for (int i = 1; i <= rowNum; i++) {

				// 行编号
				int rNum = i;
				Row row = sheet.getRow(i);
				log.info(">>>当前导入第{}行<<<", i);

				if (row == null) {
					ErrorInfoVo errorInfoVo = new ErrorInfoVo();
					errorInfoVo.setRowNum(rNum);
					errorInfoVo.setColumnName(null);
					errorInfoVo.setContent(null);
					errorInfoVo.setReason("当前行没有数据信息");
					errorList.add(errorInfoVo);
					continue;
				}

				int colNum = row.getLastCellNum();
				UserDTO userDTO = new UserDTO();

				userDTO.setDeptId(sysDept.getDeptId());
				if(ArrayUtils.contains(sysDept.getSubjectType(),SubjectTypeEnum.TYPE_SHUIBANGYUN.getCode())){
					userDTO.setCertStatus(TaxHelperCertStatus.UN_CERT.getCode());
				}
//				if(ArrayUtils.contains(sysDept.getSubjectType(),SubjectTypeEnum.TYPE_RENLIJIA.getCode())){
//					userDTO.setRljCertStatus(DingEnum.CER_STATUS_4.getType());
//				}
//				if ("1".equals(sysDept.getSubjectType())) {
//					userDTO.setCertStatus(TaxHelperCertStatus.UN_CERT.getCode());
//					userDTO.setUserType("1");
//				} else {
//					userDTO.setUserType("0");
//				}
				userDTO.setCreateTime(LocalDateTime.now());

				for (int j = 0; j < colNum; j++) {//列数据

					// 获取列名称
					String columnName = row0.getCell(j).getStringCellValue();
					int columnNum = j + 1;

					if (null == row.getCell(j)) {
						ErrorInfoVo errorInfoVo = new ErrorInfoVo();
						errorInfoVo.setRowNum(rNum);
						errorInfoVo.setColumnNum(columnNum);
						errorInfoVo.setColumnName(columnName);
						errorInfoVo.setContent(null);
						errorInfoVo.setReason("列数据为空");
						errorList.add(errorInfoVo);
						continue;
					}

					// 格式化字符串
					row.getCell(j).setCellType(CellType.STRING);
					String value = row.getCell(j).getStringCellValue()
							.replaceAll(" ", "")
							.replaceAll(",", "")
							.replaceAll("\\u00A0", "")
							.replaceAll((char) 12288 + "", "")
							.replace("+", "·");

					// 校验value
					if (StringUtil.isEmpty(value)) {
						ErrorInfoVo errorInfoVo = new ErrorInfoVo();
						errorInfoVo.setRowNum(rNum);
						errorInfoVo.setColumnNum(columnNum);
						errorInfoVo.setColumnName(columnName);
						errorInfoVo.setContent(value);
						errorInfoVo.setReason("数据为空");
						errorList.add(errorInfoVo);
						continue;
					}

					if (j == 0) {
						userDTO.setRealname(value);
					} else if (j == 1) {
						// 记录错误说明
						String errorInfo = null;
						// 正则校验手机号
						if (ValidateUtils.validPhoneNum("0", value)) {
							// 判断同一个dept下用户是否重复
							SysUser query = new SysUser();
							query.setUsername(value);
							query.setDeptId(userDTO.getDeptId());
							SysUser sysUser = sysUserMapper.selectUserByName(query);
							if (sysUser != null) {
								errorInfo = "手机号已被占用";
							}
						} else {
							errorInfo = "不是手机号";
						}

						// 封装错误信息
						if (StringUtils.isNotEmpty(errorInfo)) {
							ErrorInfoVo errorInfoVo = new ErrorInfoVo();
							errorInfoVo.setRowNum(rNum);
							errorInfoVo.setColumnNum(columnNum);
							errorInfoVo.setColumnName(columnName);
							errorInfoVo.setContent(value);
							errorInfoVo.setReason(errorInfo);
							errorList.add(errorInfoVo);
							continue;
						}

						userDTO.setPlatId(value);
						userDTO.setPhone(value);
						userDTO.setUsername(value);
					} else if (j == 2) {
						if ("全国".equals(value)) {
							userDTO.setProvince("");
						} else {
							userDTO.setProvince(value);
						}
					} else if (j == 3) {
						if (StringUtils.isEmpty(userDTO.getProvince())) {
							userDTO.setCity("");
						} else {
							userDTO.setCity(value);
						}
					} else if (j == 4) {
						// 声明错误信息记录
						String errorInfo = "";

						// 根据角色名称获取角色
						SysRole sysRole = sysRoleMapper.selectOne(Wrappers.<SysRole>query().eq("role_name", value));
						if (sysRole != null) {
							userDTO.setRole(Arrays.asList(sysRole.getRoleId()));
							if (userDTO.getRole().contains(2)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (r.getRoleId() == 3) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为药企管理员！";
													continue;
												}
												if (r.getRoleId() == 4) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为CSO管理员！";
													continue;
												}
												if (r.getRoleId() == 5) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为全职学术推广员！";
													continue;
												}
												if (r.getRoleId() == 6) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为兼职学术推广员！";
													continue;
												}
												if (r.getRoleId() == 7) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为财务管理员！";
													continue;
												}
											}
										}
									}
								}

							} else if (userDTO.getRole().contains(3)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (r.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为平台管理员！";
													continue;
												}
												if (r.getRoleId() == 4) {
													errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为CSO管理员！";
													continue;
												}
												if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
													if (r.getRoleId() == 5) {
														errorInfo = errorInfo + "用户已存在CSO关系角色,不能同时为药企管理员！";
														continue;
													}
												}
											}
										}
									}
								}

							} else if (userDTO.getRole().contains(4)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (sysRole.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在平台管理员角色,不能同时为！";
													continue;
												}

												if (r.getRoleId() == 3) {
													errorInfo = errorInfo + "用户已存在药企管理员角色,不能同时为CSO管理员！";
													continue;
												}
												if (r.getRoleId() == 5) {
													SysUserRole sysUserRole = new SysUserRole();
													sysUserRole.setUserId(userVO.getUserId());
													List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
													for (int k = 0; k < sysUserRoleList.size(); k++) {
														SysUser sysUser = userService.getById(sysUserRoleList.get(k).getUserId());
														if (!sysUser.getDeptId().equals(userDTO.getDeptId())) {
															errorInfo = errorInfo + "用户已存在全职学术推广员角色,不能为其他机构CSO管理员！";
															continue;
														}
													}
												}
											}
										}
									}
								}

							} else if (userDTO.getRole().contains(7)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (r.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在平台管理员,不能同时为财务管理员！";
													continue;
												}
												if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
													if (r.getRoleId() == 3) {
														errorInfo = errorInfo + "用户已存在药企管理员,不能同时为CSO机构财务管理员！";
														continue;
													}
												}
											}
										}
									}
								}

							} else if (userDTO.getRole().contains(5)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (r.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在平台管理员,不能同时为全职学术推广员！";
													continue;
												}
												if (r.getRoleId() == 3) {
													if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
														errorInfo = errorInfo + "用户已存在药企管理员,不能为CSO的全职学术推广员！";
														continue;
													}
												}
												if (r.getRoleId() == 4) {
													if (sysDept.getLevel() == 3 || sysDept.getLevel() == 4) {
														errorInfo = errorInfo + "用户已存在CSO管理员,不能为其他CSO的全职学术推广员！";
														continue;
													}
												}

											}
										}
									}
								}

							} else if (userDTO.getRole().contains(6)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (r.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在平台管理员,不能同时为兼职学术推广员！";
													continue;
												}
												SysUserRole sysUserRole = new SysUserRole();
												sysUserRole.setUserId(userVO.getUserId());
												List<SysUserRole> sysUserRoleList = sysUserRoleMapper.selectById(sysUserRole);
												for (int k = 0; k < sysUserRoleList.size(); k++) {
													if (sysUserRoleList.get(k).getRoleId().equals(7)) {
														SysUser sysUser = userService.getById(sysUserRoleList.get(k).getUserId());
														if (sysUser.getDeptId().equals(userDTO.getDeptId())) {
															errorInfo = errorInfo + "用户已存在财务管理员角色，不能同时为兼职学术推广员！";
															continue;
														}
													}
												}
											}
										}
									}
								}

							} else if (userDTO.getRole().contains(9)) {
								UserDTO tmp = new UserDTO();
								tmp.setUsername(userDTO.getUsername());
								tmp.setDelFlag("0");
								List<UserVO> userVoList = sysUserMapper.selectByPhone(tmp);
								if (userVoList.size() > 0) {
									for (UserVO userVO : userVoList) {
										if (null != userVO.getRoleList()) {
											for (SysRole r : userVO.getRoleList()) {
												if (sysRole.getRoleId() == 2) {
													errorInfo = errorInfo + "用户已存在平台管理员,不能同时为CRO管理员！";
													continue;
												}
												if (r.getRoleId() == 3) {
													errorInfo = errorInfo + "用户已存在药企管理员,不能同时为CRO管理员！";
													continue;
												}
											}
										}
									}
								}
							}
							if (userDTO.getRole().contains(3)) {
								if (sysDept.getLevel() != 2) {
									errorInfo = errorInfo + "角色选为药企管理员时,组织架构必须为药企！";
								}
							}
							if (!userDTO.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
								if (userDTO.getRole().contains(5) || userDTO.getRole().contains(6)) {
									errorInfo = errorInfo + "不能跨机构维护学术推广员！";
								}
							}
						}

						// 存在错误
						if (StringUtils.isNotEmpty(errorInfo) || sysRole == null) {
							ErrorInfoVo errorInfoVo = new ErrorInfoVo();
							errorInfoVo.setRowNum(rNum);
							errorInfoVo.setColumnNum(columnNum);
							errorInfoVo.setColumnName(columnName);
							errorInfoVo.setContent(errorInfo);
							errorInfoVo.setReason("角色信息有误");
							errorList.add(errorInfoVo);
							continue;
						}
					} else if (j == 5) {
						if (!"活跃".equals(value) && !"休眠".equals(value)) {
							ErrorInfoVo errorInfoVo = new ErrorInfoVo();
							errorInfoVo.setRowNum(rNum);
							errorInfoVo.setColumnNum(columnNum);
							errorInfoVo.setColumnName(columnName);
							errorInfoVo.setContent(value);
							errorInfoVo.setReason("角色状态有误");
							errorList.add(errorInfoVo);
							continue;
						}
						userDTO.setLockFlag("活跃".equals(value) ? "0" : "9");
					} else if (j == 6) {
						userDTO.setProvinceCode(value);
					} else if (j == 7) {
						userDTO.setCityCode(value);
					}
				}

				// 记录争取额的信息
				userDTOList.add(userDTO);
			}

			// 判断是否有错误的数据行，如果没有，执行入库操作
			if (CollectionUtils.isEmpty(errorList)) {
				userDTOList.forEach(userDTO -> {
					userService.saveUser(userDTO);
				});
			}
			return R.ok(errorList);

		} catch (Exception e) {
			log.error("批量创建用户失败:{}",e.getMessage());
			e.printStackTrace();
			return R.failed("批量创建用户失败");
		}
	}

	/**
	 * 更新用户信息
	 *
	 * @param userDto 用户信息
	 * @return R
	 */
	@SysLog("更新用户信息")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('sys_user_edit')")
	public R updateUser(@Valid @RequestBody UserDTO userDto) {

		// TODO qcc调用企查查查询是否为主要人员
		/**
		 * @Modify: start qcc调用企查查查询是否为主要人员
		 * @Version: v2021.4.16
		 * @Author: ryz
		 * @Date: 2021/5/6
		 */
		SysDept sysDept = sysDeptService.getById(userDto.getDeptId());


		if (null != userDto.getUserId()) {
			SysUser oldData = userService.getById(userDto.getUserId());
			if (!userDto.getRealname().equals(oldData.getRealname())) {
				if (userDto.getRole().contains(6) || userDto.getRole().contains(10)) {
					Map<String, String> params = new HashMap<>();
					params.put("searchKey", sysDept.getName());
					params.put("pageIndex", "1");
					params.put("pageSize", "10");
					EmployeeGetListResponse qccRes = qccService.getList(params);
					if (qccRes.isSuccess()) {
						for (EmployeeGetListDataItem item : qccRes.getResult()) {
							if (userDto.getRealname().equals(item.getName())) {
								if(userDto.getRole().contains(10)){
									return R.failed(String.format("%s为该公司%s，不能为兼职CRC",item.getName(),item.getJob()));
								}else{
									return R.failed(String.format("%s为该公司%s，不能为兼职学术推广员",item.getName(),item.getJob()));
								}

							}
						}
						if(null != qccRes.getPaging()){
							PageInfo qccPage = qccRes.getPaging();
							// 需要分页查询
							if (qccPage.getTotalRecords() > qccPage.getPageIndex() * qccPage.getPageSize()) {
								int totalPage = qccPage.getTotalRecords() / qccPage.getPageSize();

								for (int i = 2; i < totalPage + 2; i++) {
									params.put("pageIndex", i + "");
									qccRes = qccService.getList(params);
									if (qccRes.isSuccess()) {
										for (EmployeeGetListDataItem item : qccRes.getResult()) {
											if(userDto.getRole().contains(10)){
												return R.failed(String.format("%s为该公司%s，不能为兼职CRC",item.getName(),item.getJob()));
											}else{
												return R.failed(String.format("%s为该公司%s，不能为兼职学术推广员",item.getName(),item.getJob()));
											}
										}
									} else {
										break;
									}
								}
							}
						}

					}
				}
			}


			List<Integer> cetList = new ArrayList<>();
			cetList.add(1);
			cetList.add(3);
			cetList.add(11);
			cetList.add(12);
			cetList.add(13);
			cetList.add(14);
			cetList.add(15);
			cetList.add(16);

			/**
			 * @Modify: start
			 * @Version: v2021.4.16
			 * @Author: ryz
			 * @Date: 2021/4/29
			 * 去掉修改用户信息时候的认证
			 */
			boolean updateFlag = true;
			if (null != oldData.getCertStatus()) {
				if (oldData.getCertStatus() != 10) {
					updateFlag = false;
				}

			}
			if (null != oldData.getRljCertStatus()) {
				if (oldData.getRljCertStatus() == 1 || oldData.getRljCertStatus() == 3) {
					updateFlag = false;
				}
			}
			if (!updateFlag) {
				if (!userDto.getRealname().equals(oldData.getRealname())) {
					return R.failed("用户认证通过或正在认证，暂不能修改姓名");
				}
				if (!userDto.getIdCardNumber().equals(oldData.getIdCardNumber())) {
					return R.failed("用户认证通过或正在认证，暂不能修改身份证号");
				}
			}

//			if (null != oldData.getCertStatus() && cetList.contains(oldData.getCertStatus())) {
//				if (!userDto.getRealname().equals(oldData.getRealname())) {
//					return R.failed("用户认证通过或正在认证，暂不能修改姓名");
//				}
//				if (!userDto.getIdCardNumber().equals(oldData.getIdCardNumber())) {
//					return R.failed("用户认证通过或正在认证，暂不能修改身份证号");
//				}
//			} else {
//				if (null != userDto.getDeptId() && StringUtils.isNotEmpty(userDto.getIdCardNumber())) {
//					SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
//					if (null != sysDept && StringUtils.isNotEmpty(sysDept.getAppId())) {
//						if ("1".equals(oldData.getUserType())) {
//							TaxHelperConfig config = new TaxHelperConfig();
//							config.setAppId(sysDept.getAppId());
//							config.setAppSecrete(sysDept.getAppSecret());
//							config.setQueryUrl(sysDept.getQueryUrl());
//
//							TaxHelperAddMemberModel model = new TaxHelperAddMemberModel(config);
//							model.setIdCard(userDto.getIdCardNumber());
//							model.setPhone(userDto.getUsername());
//							model.setName(userDto.getRealname());
//							model.setNotifyUrl("http://localhost:8808");
//							String entCode = null;
////							switch (sysDept.getLevel()) {
////								case 1:
////									entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel1Id(), 1);
////									break;
////								case 2:
////									entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel2Id(), 2);
////									break;
////								case 3:
////									entCode = sysDeptService.selectEntCodeByLevelId(sysDept.getLevel3Id(), 3);
////									break;
////							}
//
//							WmDaAgent agent = wmDaAgentService.selectByDeptId(sysDept.getDeptId());
//							WmDaDrugEnt drugEnt = wmDaDrugEntService.selectByDeptId(sysDept.getDeptId());
//							if (null != agent) {
//								model.setEnterpriseCode(agent.getZjNumber());
//							} else {
//								model.setEnterpriseCode(drugEnt.getEntcode());
//							}
//							if (StringUtils.isEmpty(model.getEnterpriseCode())) {
//								return R.failed("未配置企业的统一社会信用代码");
//							}
//							TaxHelperAddMemberRequest request = new TaxHelperAddMemberRequest(model);
//							TaxHelperResponse response = taxHelperService.addMembers(request);
//
//							// 记录认证结果
//							SysCertResultRecord record = new SysCertResultRecord();
//							record.setCode(response.getCode());
//							record.setCreateTime(LocalDateTime.now());
//							record.setDeptId(sysDept.getDeptId());
//							record.setMessage(StringUtils.isNotEmpty(response.getMessage())?response.getMessage():"无");
//							record.setStatus(null);
//							record.setSubjectType(1);
//							record.setMethod("增员");
//							record.setUserId(oldData.getUserId());
//							sysCertResultRecordMapper.insert(record);
//
//							if (response.success()) {
//								userDto.setCertStatus(TaxHelperCertStatus.ADD_MEMBER.getCode());
//							} else {
//								log.info("税邦云增员失败：{},{}，原因是：{}", userDto.getRealname(), userDto.getUsername(),
//										response.getMessage());
//								userDto.setIdCardNumber(null);
//							}
//
//						} else {
//							DingConfig config = new DingConfig();
//							config.setAppId(sysDept.getAppId());
//							config.setAppSecret(sysDept.getAppSecret());
//							config.setQueryUrl(sysDept.getQueryUrl());
//
//							Map map = new HashMap();
//							map.put("name", userDto.getRealname());
//							map.put("mobile", userDto.getUsername());
//							map.put("idCard", userDto.getIdCardNumber());
//							map.put("idType", "1");
//							List rzlist = new ArrayList();
//							rzlist.add(map);
//
//							EmployCheckResponse response = dingService.check(config, rzlist);
//							if (!response.isSuccess()) {
//								return R.failed("人力家接口调用失败");
//							}
//							// 接口请求成功，但是接口返回的状态还需区分checkStatus，1:验证通过，状态改成已提交，2:验证不通过，状态改成认证失败
//							List<EmployCheckResultItem> freelancesResult = response.getData().getFreelancesResult();
//							if (CollectionUtils.isNotEmpty(freelancesResult)) {
//								for (EmployCheckResultItem employCheckResultItem : freelancesResult) {
//									if (employCheckResultItem.getIdCard().equals(userDto.getIdCardNumber())) {
//										if (employCheckResultItem.getCheckStatus() == 1) {// 验证通过
//											userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//											continue;
//										} else {// 验证不通过
//											if ("1014".equals(employCheckResultItem.getCheckRemark())) {
//												log.info("认证返回1014按成功处理：{},{}", userDto.getRealname(),
//														userDto.getUsername());
//												userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//												continue;
//											}
//											if ("1018".equals(employCheckResultItem.getCheckRemark())) {
//												log.info("认证返回1018按处理中处理：{},{}", userDto.getRealname(),
//														userDto.getUsername());
//												userDto.setCertStatus(DingEnum.CER_STATUS_3.getType());
//												continue;
//											}
//											if ("1019".equals(employCheckResultItem.getCheckRemark())) {
//												log.info("认证返回1019按成功处理：{},{}", userDto.getRealname(),
//														userDto.getUsername());
//												userDto.setCertStatus(DingEnum.CER_STATUS_1.getType());
//												continue;
//											}
//											if ("1020".equals(employCheckResultItem.getCheckRemark())) {
//												log.info("认证返回1020按认证失败处理：{},{}", userDto.getRealname(),
//														userDto.getUsername());
//												userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//												continue;
//											}
//											userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//											log.info("认证失败:" + employCheckResultItem.getCheckRemark());
//										}
//									}
//								}
//							} else {
//								userDto.setCertStatus(DingEnum.CER_STATUS_2.getType());
//							}
//						}
//					}
//				}
//			}
		}

		if (SecurityUtils.getRoles().contains(2) || SecurityUtils.getRoles().contains(1)) {
			if (userDto.getRole().contains(2)) {
				UserDTO userDTO = new UserDTO();
				userDTO.setPlatId(userDto.getUsername());
				userDTO.setDelFlag("0");
				List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
				if (userDTOList.size() > 0) {
					for (UserVO userVO : userDTOList) {
						if (null != userVO.getRoleList()) {
							for (SysRole sysRole : userVO.getRoleList()) {
								if (sysRole.getRoleId() == 3) {
									return R.failed("用户不能同时为平台管理员和药企管理员");
								}
							}
						}
					}
				}

			} else if (userDto.getRole().contains(3)) {
				UserDTO userDTO = new UserDTO();
				userDTO.setPlatId(userDto.getUsername());
				userDTO.setDelFlag("0");
				List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
				if (userDTOList.size() > 0) {
					for (UserVO userVO : userDTOList) {
						if (null != userVO.getRoleList()) {
							for (SysRole sysRole : userVO.getRoleList()) {
								if (sysRole.getRoleId() == 2) {
									return R.failed("用户不能同时为平台管理员和药企管理员");
								}
								if (userDto.getDeptId().equals(userVO.getDeptId())) {
									if (sysRole.getRoleId() == 6) {
										return R.failed("用户已存在本机构药企管理员,不能同时为兼职学术推广员");
									}
								}
							}
						}
					}
				}
				SysUser query = new SysUser();
				query.setUsername(userDto.getUsername());
				query.setDeptId(userDto.getDeptId());
				SysUser sysUser = sysUserMapper.selectUserByName(query);
				if (sysUser != null) {
//					SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());
					if (sysDept.getLevel() != 2) {
						return R.failed("角色选为药企管理员时,组织架构必须为药企");
					}
				}

			} else if (userDto.getRole().contains(4)) {
				UserDTO userDTO = new UserDTO();
				userDTO.setPlatId(userDto.getUsername());
				userDTO.setDelFlag("0");
				List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
				if (userDTOList.size() > 0) {
					for (UserVO userVO : userDTOList) {
						if (null != userVO.getRoleList()) {
							for (SysRole sysRole : userVO.getRoleList()) {
								if (sysRole.getRoleId() == 2) {
									return R.failed("用户不能同时为平台管理员和经销商管理员");
								}

								if (sysRole.getRoleId() == 3) {
									return R.failed("用户不能同时为药企管理员和经销商管理员");
								}
								if(!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 4){
									return R.failed("用户已存在CSO管理员角色,不能维护为其他经销商组织的CSO管理员角色");
								}
								if(!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 9){
									return R.failed("用户已存在CRO管理员角色,不能维护为其他经销商组织的CSO管理员角色");
								}
							}
						}
					}
				}

			} else if (userDto.getRole().contains(9)) {
				UserDTO userDTO = new UserDTO();
				userDTO.setPlatId(userDto.getUsername());
				userDTO.setDelFlag("0");
				List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
				if (userDTOList.size() > 0) {
					for (UserVO userVO : userDTOList) {
						if (null != userVO.getRoleList()) {
							for (SysRole sysRole : userVO.getRoleList()) {
								if (sysRole.getRoleId() == 2) {
									return R.failed("用户不能同时为平台管理员和CRO管理员");
								}

								if (sysRole.getRoleId() == 3) {
									return R.failed("用户不能同时为药企管理员和CRO管理员");
								}
								if(!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 4){
									return R.failed("用户已存在CSO管理员角色,不能维护为其他经销商组织的CRO管理员角色");
								}
								if(!userDto.getUserId().equals(userVO.getUserId()) && sysRole.getRoleId() == 9){
									return R.failed("用户已存在CRO管理员角色,不能维护为其他经销商组织的CRO管理员角色");
								}
							}
						}
					}
				}

			}else if (userDto.getRole().contains(6)) {
				UserDTO userDTO = new UserDTO();
				userDTO.setPlatId(userDto.getUsername());
				userDTO.setDelFlag("0");
				List<UserVO> userDTOList = userService.getUserVoByPlatId(userDTO);
				if (userDTOList.size() > 0) {
					for (UserVO userVO : userDTOList) {
						if (null != userVO.getRoleList()) {
							for (SysRole sysRole : userVO.getRoleList()) {
								if (sysRole.getRoleId() == 2) {
									return R.failed("用户不能同时为平台管理员和兼职学术推广员");
								}
								if (userDto.getDeptId().equals(userVO.getDeptId())) {
									if (sysRole.getRoleId() == 3) {
										return R.failed("用户已存在本药企药企管理员,不能同时为兼职学术推广员");
									}
								}
							}
						}
					}
				}

			} else {
				// SysUser sysUser = new SysUser();
				//// SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
				//// sysUser.setDeptId(sysDept.getDeptId());
				//// sysUser.setUsername(userDto.getUsername());
				////
				//// List<SysUser> sysUserList = userService.list(Wrappers.query(sysUser));
				//// if (sysUserList.size() > 1) {
				//// return R.failed("用户信息已存在");
				//// }
				if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
					if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
						return R.failed("不能跨机构维护学术推广员");
					}
				}
			}

		} else {
			// SysUser sysUser = new SysUser();
			// SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
			// sysUser.setDeptId(sysDept.getDeptId());
			// sysUser.setUsername(userDto.getUsername());
			//
			// List<SysUser> sysUserList = userService.list(Wrappers.query(sysUser));
			// if (sysUserList.size() > 1) {
			// return R.failed("用户信息已存在");
			// }
			if (!userDto.getDeptId().equals(SecurityUtils.getUser().getDeptId())) {
				if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
					return R.failed("不能跨机构维护学术推广员");
				}
			}

			SysUser query = new SysUser();
			query.setUsername(userDto.getUsername());
			query.setDeptId(userDto.getDeptId());
			SysUser sysUser = sysUserMapper.selectUserByName(query);
			if (sysUser != null) {
				if (userDto.getRole().contains(6) && userDto.getRole().contains(4)) {
					return R.failed("用户不能同时为经销商管理员角色和兼职学术推广员角色");
				}
//				if (userDto.getRole().contains(7) && userDto.getRole().contains(5)) {
//					return R.failed("用户不能同时为财务管理员角色和全职学术推广员角色");
//				}
				if (userDto.getRole().contains(7) && userDto.getRole().contains(6)) {
					return R.failed("用户不能同时为财务管理员角色和兼职学术推广员角色");
				}
				if (userDto.getRole().contains(3)) {
//					SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());
					if (sysDept.getLevel() != 2) {
						return R.failed("角色选为药企管理员时,组织架构必须为药企");
					}
				}
			}
		}
		//TODO 身份证年龄
		if (StringUtils.isNotEmpty(userDto.getIdCardNumber())) {
			String str = userDto.getIdCardNumber();
			str = str.substring(6, 10);
			String str1 = userDto.getIdCardNumber();
			str1 = str1.substring(10, 12);
			String str2 = userDto.getIdCardNumber();
			str2 = str2.substring(12, 14);
			int i = Integer.parseInt(str);
			int i1 = Integer.parseInt(str1) - 1;
			int i2 = Integer.parseInt(str2);

			Calendar calendar = Calendar.getInstance();
			calendar.setTime(new Date());//放入Date类型数据

			int year = calendar.get(Calendar.YEAR);//获取年份
			int month = calendar.get(Calendar.MONTH);//获取月份
			int day = calendar.get(Calendar.DATE);//获取日

			if (year - i < 18) {
				return R.failed("年龄未满十八");
			} else if (year - i == 18) {
				if (month > i1) {
					return R.failed("年龄未满十八");
				} else if (month == i1) {
					if (day < i2) {
						return R.failed("年龄未满十八");
					}
				}
			} else if (year - i > 65) {
				return R.failed("年龄已超过六十五");
			} else if (year - i == 65) {
				if (month > i1) {
					return R.failed("年龄已超过六十五");
				} else if (month == i1) {
					if (day >= i2) {
						return R.failed("年龄已超过六十五");
					}
				}
			}
		}

		return R.ok(userService.updateUser(userDto));
	}

	/**
	 * 分页查询用户
	 *
	 * @param page    参数集
	 * @param userDTO 查询参数列表
	 * @return 用户集合
	 */
	@GetMapping("/page")
	public R getUserPage(Page page, UserDTO userDTO) {

		return R.ok(userService.getUsersWithRolePageNoLock(page, userDTO));
	}

	/**
	 * 分页查询用户
	 *
	 * @param page    参数集
	 * @param userDTO 查询参数列表
	 * @return 用户集合
	 */
	@GetMapping("/selectUserList")
	public R selectUserList(Page page, UserDTO userDTO) {
		if(!"0".equals(userDTO.getSubType()) && !"1".equals(userDTO.getSubType())){
			userDTO.setSubType(null);
		}

		IPage<UserVO> userVosPage = userService.selectUserList(page, userDTO);
		if("0".equals(userDTO.getSubType())){
			List<UserVO> userDTOList = userVosPage.getRecords();
			userDTOList.forEach(item->item.setCertStatus(null));
		}
		if("1".equals(userDTO.getSubType())){
			List<UserVO> userDTOList = userVosPage.getRecords();
			userDTOList.forEach(item->item.setRljCertStatus(null));
		}
		return R.ok(userVosPage);
	}

	/**
	 * 修改个人信息
	 *
	 * @param userDto userDto
	 * @return success/false
	 */
	@SysLog("修改个人信息")
	@PutMapping("/edit")
	public R updateUserInfo(@Valid @RequestBody UserDTO userDto) {
		userDto.setUserId(SecurityUtils.getUser().getId());
		return userService.updateUserInfo(userDto);
	}

	/**
	 * @param username 用户名称
	 * @return 上级部门用户列表
	 */
	@GetMapping("/ancestor/{username}")
	public R listAncestorUsers(@PathVariable String username) {
		return R.ok(userService.listAncestorUsers(username));
	}

	/**
	 * 修改个人信息
	 *
	 * @param userDTO userDTO
	 * @return success/false
	 */
	@SysLog("查询用户列表")
	@GetMapping("/list")
	public R listScope(Page page, UserDTO userDTO) {
		if (null != userDTO.getRole() && userDTO.getRole().size() > 0) {
			userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
			page.setSize(10000);
			List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
			List<Map<String, String>> answerList = new ArrayList<>();
			if (sysUserList.size() > 0) {
				for (UserVO sysUser : sysUserList) {
					Map<String, String> map = new HashMap<>();
					map.put("id", sysUser.getUserId() + "");
					map.put("realname", sysUser.getRealname() + "");
					answerList.add(map);
				}

			}
			return R.ok(answerList);
		} else {
			page.setSize(10000);
			List<Integer> list = new ArrayList<>();
			list.add(5);
			list.add(6);
			userDTO.setRole(list);

			List<UserVO> sysUserList = userService.getUsersWithRolePage(page, userDTO).getRecords();
			List<Map<String, String>> answerList = new ArrayList<>();
			if (sysUserList.size() > 0) {
				for (UserVO sysUser : sysUserList) {
					Map<String, String> map = new HashMap<>();
					map.put("id", sysUser.getUserId() + "");
					map.put("realname", sysUser.getRealname() + "");
					answerList.add(map);
				}

			}
			return R.ok(answerList);
		}
	}

	/**
	 * 查询用户列表
	 *
	 * @param userDTO
	 * @return
	 */
	@SysLog("查询用户列表")
	@GetMapping("/listNoScope")
	public R listNoScope(UserDTO userDTO) {

		SysUser sysUserSel = new SysUser();

		if (SecurityUtils.getRoles().contains(2) || SecurityUtils.getRoles().contains(1)) {

		} else {
			sysUserSel.setDrugEntId(SecurityUtils.getUser().getDrugEntId());
		}
		QueryWrapper<SysUser> query = Wrappers.query(sysUserSel);
		if (StringUtils.isNotEmpty(userDTO.getRealname())) {
			query.lambda().like(SysUser::getRealname, userDTO.getRealname());
		}
		List<SysUser> sysUserList = userService.list(query);
		List<Map<String, String>> answerList = new ArrayList<>();
		if (sysUserList.size() > 0) {
			for (SysUser sysUser : sysUserList) {
				Map<String, String> map = new HashMap<>();
				map.put("id", sysUser.getUserId() + "");
				map.put("realname", sysUser.getRealname() + "");
				map.put("name", sysUser.getUsername() + "");

				answerList.add(map);
			}

		}
		return R.ok(answerList);
	}

	/**
	 * 修改个人信息
	 *
	 * @param userDTO userDTO
	 * @return success/false
	 */
	@SysLog("查询用户列表")
	@GetMapping("/listNoScopeAll")
	public R listNoScopeAll(Page page, UserDTO userDTO) {
		page.setSize(10000);
		QueryWrapper<SysUser> queryWrapper = null;
		if (userDTO.getDeptId() != null){
			SysUser sysUser = new SysUser();
			sysUser.setDeptId(userDTO.getDeptId());
			queryWrapper = Wrappers.query(sysUser);
		} else {
			queryWrapper = Wrappers.query();
		}

		List<SysUser> sysUserList = userService.list(queryWrapper);
		List<Map<String, String>> answerList = new ArrayList<>();
		if (sysUserList.size() > 0) {
			for (SysUser sysUser : sysUserList) {
				Map<String, String> map = new HashMap<>();
				map.put("id", sysUser.getUserId() + "");
				map.put("realname", sysUser.getRealname() + "");
				map.put("name", sysUser.getUsername() + "");

				answerList.add(map);
			}

		}
		return R.ok(answerList);
	}

}
