/*
 *
 *      Copyright (c) 2018-2025, hnqz All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the pig4cloud.com developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: hnqz
 *
 */

package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.nacos.common.utils.Md5Utils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.UserDTO;
import com.qunzhixinxi.hnqz.admin.api.dto.UserInfo;
import com.qunzhixinxi.hnqz.admin.api.dto.UserPwd;
import com.qunzhixinxi.hnqz.admin.api.entity.*;
import com.qunzhixinxi.hnqz.admin.api.vo.MenuVO;
import com.qunzhixinxi.hnqz.admin.api.vo.UserVO;
import com.qunzhixinxi.hnqz.admin.entity.SysCertResultRecord;
import com.qunzhixinxi.hnqz.admin.entity.SysDeptSub;
import com.qunzhixinxi.hnqz.admin.entity.input.UserCertificationInput;
import com.qunzhixinxi.hnqz.admin.entity.output.UserCertificationOutput;
import com.qunzhixinxi.hnqz.admin.enums.SubjectTypeEnum;
import com.qunzhixinxi.hnqz.admin.mapper.*;
import com.qunzhixinxi.hnqz.admin.service.*;
import com.qunzhixinxi.hnqz.common.core.constant.CacheConstants;
import com.qunzhixinxi.hnqz.common.core.constant.CommonConstants;
import com.qunzhixinxi.hnqz.common.core.util.R;
import com.qunzhixinxi.hnqz.common.data.datascope.DataScope;
import com.qunzhixinxi.hnqz.common.ding.config.DingConfig;
import com.qunzhixinxi.hnqz.common.ding.entity.EmployCheckResponse;
import com.qunzhixinxi.hnqz.common.ding.entity.EmployCheckResultItem;
import com.qunzhixinxi.hnqz.common.ding.enums.DingEnum;
import com.qunzhixinxi.hnqz.common.ding.service.DingService;
import com.qunzhixinxi.hnqz.common.security.util.SecurityUtils;
import com.qunzhixinxi.hnqz.common.taxhelper.config.TaxHelperConfig;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.model.TaxHelperAddMemberModel;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.request.TaxHelperAddMemberRequest;
import com.qunzhixinxi.hnqz.common.taxhelper.entity.response.TaxHelperResponse;
import com.qunzhixinxi.hnqz.common.taxhelper.enums.TaxHelperCertStatus;
import com.qunzhixinxi.hnqz.common.taxhelper.service.TaxHelperService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Caching;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hnqz
 * @date 2017/10/31
 */
@Slf4j
@Service
@AllArgsConstructor
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

	private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();

	private final SysMenuService sysMenuService;

	private final SysRoleService sysRoleService;

	private final SysDeptService sysDeptService;

	private final SysUserRoleService sysUserRoleService;

	private final SysUserRoleMapper sysUserRoleMapper;

	private final SysUserMapper sysUserMapper;

	private final SysDeptRelationService sysDeptRelationService;

	private final SysDeptRelationMapper sysDeptRelationMapper;

	private final RedisTemplate redisTemplate;

	private final DingService dingService;

	private final SysDeptSubMapper sysDeptSubMapper;

	private final TaxHelperService taxHelperService;

	private final WmDaAgentMapper wmDaAgentMapper;

	private final WmDaDrugEntMapper wmDaDrugEntMapper;

	private final SysCertResultRecordMapper sysCertResultRecordMapper;

	/**
	 * @Modify: start
	 * @Version: v2021.4.16
	 * @Author: ryz
	 * @Date: 2021/5/6
	*/
	@Override
	public R getAuthInfo(UserCertificationInput input) {

		SysUser sysUser = sysUserMapper.selectById(input.getUserId());

		SysDeptSub sysDeptSub = new SysDeptSub();
//		sysDeptSub.setDeptId(SecurityUtils.getUser().getDeptId());
		sysDeptSub.setDeptId(sysUser.getDeptId());
		sysDeptSub.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		List<SysDeptSub> subList = sysDeptSubMapper.selectList(Wrappers.query(sysDeptSub));
		UserCertificationOutput output = new UserCertificationOutput();
		output.setSubList(subList);
		output.setUserInfo(sysUser);
		return R.ok(output);
	}

	/**
	 * @Modify: start
	 * @Version: 2021.4.16
	 * @Author: ryz
	 * @Date: 2021/4/21
	 * @Desc: 认证方式 0-人力家 1-税邦云
	 */
	@Override
	@Transactional
	public R batchChannelCert(UserCertificationInput input) {
		SysUser sysUser = baseMapper.selectById(input.getUserId());
		String inputIdCardNumber = input.getIdCardNumber();
		sysUser.setIdCardNumber(inputIdCardNumber);
		SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());

		StringBuilder msg = new StringBuilder();

		for (String subType : input.getSubList()) {
			//人力家
			if (SubjectTypeEnum.TYPE_RENLIJIA.getCode().equals(subType)) {
				Map<String, String> resultMap = certByRenLiJia(sysUser, sysDept);
				msg.append("人力家认证结果:");
				msg.append(resultMap.get("msg"));
				msg.append(";");
			} else if (SubjectTypeEnum.TYPE_SHUIBANGYUN.getCode().equals(subType)) {
				Map<String, String> resultMap = certByShuiBangYun(sysUser, sysDept);
				msg.append("税邦云认证结果:");
				msg.append(resultMap.get("msg"));
				msg.append(";");
			}
		}
		String resultMsg = msg.toString();

		baseMapper.updateById(sysUser);
		return R.ok(null, resultMsg.substring(0,resultMsg.length()-1));
	}

	private Map<String, String> certByShuiBangYun(SysUser sysUser, SysDept sysDept) {
		Map<String, String> resultMap = new HashMap<>(2);
		resultMap.put("msg", "增员失败");
		SysDeptSub subCondition = new SysDeptSub();
		subCondition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		subCondition.setDeptId(sysDept.getDeptId());
		subCondition.setSubjectType(SubjectTypeEnum.TYPE_SHUIBANGYUN.getCode());
		SysDeptSub deptSub = sysDeptSubMapper.selectOne(Wrappers.query(subCondition));
		if (null == deptSub || StringUtils.isBlank(deptSub.getAppId())) {
			resultMap.put("msg", "人员所在机构未配置结算信息");
			return resultMap;
		}
		log.info("------开始认证------");
		log.info("------认证渠道：税邦云------");
		log.info("------认证人员姓名：------ {}", sysUser.getRealname());
		log.info("------认证人员手机号：------ {}", sysUser.getUsername());
		log.info("------认证人员身份证：------ {}", sysUser.getIdCardNumber());

		TaxHelperConfig config = new TaxHelperConfig();
		config.setAppId(deptSub.getAppId());
		config.setAppSecrete(deptSub.getAppSecret());
		config.setQueryUrl(deptSub.getQueryUrl());

		TaxHelperAddMemberModel model = new TaxHelperAddMemberModel(config);
		model.setIdCard(sysUser.getIdCardNumber());
		model.setPhone(sysUser.getUsername());
		model.setType(Integer.parseInt(deptSub.getSubjectUserType()));
		model.setName(sysUser.getRealname());
		model.setNotifyUrl("http://localhost:8808");

//		WmDaAgent agent = wmDaAgentMapper.selectByDeptId(sysDept.getDeptId());
//		WmDaDrugEnt drugEnt = wmDaDrugEntMapper.selectByDeptId(sysDept.getDeptId());
//		if (null != agent) {
//			model.setEnterpriseCode(agent.getZjNumber());
//		} else {
//			model.setEnterpriseCode(drugEnt.getEntcode());
//		}
		model.setEnterpriseCode(sysDept.getTaxCode());
		if (StringUtils.isEmpty(model.getEnterpriseCode())) {

			resultMap.put("msg", "未配置企业的统一社会信用代码");
			return resultMap;
		}

		TaxHelperAddMemberRequest request = new TaxHelperAddMemberRequest(model);
		TaxHelperResponse response = taxHelperService.addMembers(request);

		// 记录认证结果
		SysCertResultRecord record = new SysCertResultRecord();
		record.setCode(response.getCode());
		record.setCreateTime(LocalDateTime.now());
		record.setDeptId(sysDept.getDeptId());
		record.setMessage(StringUtils.isNotEmpty(response.getMessage()) ? response.getMessage() : "无");
		record.setStatus(null);
		record.setSubjectType(1);
		record.setMethod("增员");
		record.setUserId(sysUser.getUserId());
		sysCertResultRecordMapper.insert(record);

		if (response.success()) {
			sysUser.setCertStatus(TaxHelperCertStatus.ADD_MEMBER.getCode());
			resultMap.put("msg", "增员成功");
		} else {
			log.info("税邦云增员失败：{},{}，原因是：{}", sysUser.getRealname(), sysUser.getUsername(),
					response.getMessage());
			resultMap.put("msg", "增员失败");
		}
		return resultMap;
	}

	private Map<String, String> certByRenLiJia(SysUser sysUser, SysDept sysDept) {
		Map<String, String> resultMap = new HashMap<>(2);
		resultMap.put("msg", "人力家接口调用失败");
		SysDeptSub subCondition = new SysDeptSub();
		subCondition.setEnableFlag(SubjectTypeEnum.ENABLE_FLAG_TRUE.getCode());
		subCondition.setDeptId(sysDept.getDeptId());
		subCondition.setSubjectType(SubjectTypeEnum.TYPE_RENLIJIA.getCode());
		SysDeptSub deptSub = sysDeptSubMapper.selectOne(Wrappers.query(subCondition));
		if (null == deptSub || StringUtils.isBlank(deptSub.getAppId())) {
			resultMap.put("msg", "人员所在机构未配置结算信息");
			return resultMap;
		}
		log.info("------开始认证------");
		log.info("------认证渠道：人力家------");
		log.info("------认证人员姓名：------ {}", sysUser.getRealname());
		log.info("------认证人员手机号：------ {}", sysUser.getUsername());
		log.info("------认证人员身份证：------ {}", sysUser.getIdCardNumber());

		DingConfig config = new DingConfig();
		config.setAppId(deptSub.getAppId());
		config.setAppSecret(deptSub.getAppSecret());
		config.setQueryUrl(deptSub.getQueryUrl());

		Map map = new HashMap();
		map.put("name", sysUser.getRealname());
		map.put("mobile", sysUser.getUsername());
		map.put("idCard", sysUser.getIdCardNumber());
		map.put("idType", "1");
		List certList = new ArrayList();
		certList.add(map);

		EmployCheckResponse response = dingService.check(config, certList);
		if (!response.isSuccess()) {
			resultMap.put("msg", "人力家接口调用失败");
			return resultMap;
		}
		// 接口请求成功，但是接口返回的状态还需区分checkStatus，1:验证通过，状态改成已提交，2:验证不通过，状态改成认证失败
		List<EmployCheckResultItem> freelancesResult = response.getData().getFreelancesResult();
		if (CollectionUtils.isNotEmpty(freelancesResult)) {
			for (EmployCheckResultItem employCheckResultItem : freelancesResult) {
				if (employCheckResultItem.getIdCard().equals(sysUser.getIdCardNumber())) {
					// 验证通过
					if (employCheckResultItem.getCheckStatus() == 1) {
						sysUser.setRljCertStatus(DingEnum.CER_STATUS_3.getType());
						resultMap.put("msg", "认证提交成功");
						continue;
					} else {// 验证不通过
						if ("1014".equals(employCheckResultItem.getCheckRemark())) {
							log.info("认证返回1014按成功处理：{},{}", sysUser.getRealname(),
									sysUser.getUsername());
							sysUser.setRljCertStatus(DingEnum.CER_STATUS_3.getType());
							resultMap.put("msg", "认证提交成功");
							continue;
						}
						if ("1018".equals(employCheckResultItem.getCheckRemark())) {
							log.info("认证返回1018按处理中处理：{},{}", sysUser.getRealname(),
									sysUser.getUsername());
							sysUser.setRljCertStatus(DingEnum.CER_STATUS_3.getType());
							resultMap.put("msg", "认证提交成功");
							continue;
						}
						if ("1019".equals(employCheckResultItem.getCheckRemark())) {
							log.info("认证返回1019按成功处理：{},{}", sysUser.getRealname(),
									sysUser.getUsername());
							sysUser.setRljCertStatus(DingEnum.CER_STATUS_1.getType());
							resultMap.put("msg", "认证成功");
							continue;
						}
						if ("1020".equals(employCheckResultItem.getCheckRemark())) {
							log.info("认证返回1020按认证失败处理：{},{}", sysUser.getRealname(),
									sysUser.getUsername());
							sysUser.setRljCertStatus(DingEnum.CER_STATUS_2.getType());
							resultMap.put("msg", "认证失败");
							continue;
						}
						sysUser.setRljCertStatus(DingEnum.CER_STATUS_2.getType());
						resultMap.put("msg", "认证失败");
						log.info("认证失败:" + employCheckResultItem.getCheckRemark());
					}
				}
			}
		} else {
			sysUser.setRljCertStatus(DingEnum.CER_STATUS_2.getType());
			resultMap.put("msg", "认证失败");
		}

		return resultMap;
	}

	/**
	 * 保存用户信息
	 *
	 * @param userDto DTO 对象
	 * @return success/fail
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean saveUser(UserDTO userDto) {


		SysUser sysUser = new SysUser();
		if (null != userDto.getUsername()) {
			sysUser.setPassword(userDto.getUsername().substring(5));
			userDto.setPassword(userDto.getUsername().substring(5));
		}
		BeanUtils.copyProperties(userDto, sysUser);
		sysUser.setDelFlag(CommonConstants.STATUS_NORMAL);
		sysUser.setW1(Md5Utils.getMD5(userDto.getPassword().getBytes()));
		sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
		sysUser.setPlatId(userDto.getPhone());

		//TODO 添加人员增加认证信息  身份证 实名认证状态


		baseMapper.insert(sysUser);

		if (userDto.getRole().contains(5) || userDto.getRole().contains(6)) {
			redisTemplate.opsForValue().set("user_add_notice_key:" + sysUser.getUserId(), sysUser.getUserId());
		}

		List<SysUserRole> userRoleList = userDto.getRole().stream().map(roleId -> {
			SysUserRole userRole = new SysUserRole();
			userRole.setUserId(sysUser.getUserId());
			userRole.setRoleId(roleId);
			return userRole;
		}).collect(Collectors.toList());
		System.out.println("-----" + userRoleList.size());
		System.out.println("-------" + userRoleList.toString());
		return sysUserRoleService.saveBatch(userRoleList);
	}

	/**
	 * 通过查用户的全部信息
	 *
	 * @param sysUser 用户
	 * @return
	 */
	@Override
	public UserInfo findUserInfo(SysUser sysUser) {
		UserInfo userInfo = new UserInfo();
		userInfo.setSysUser(sysUser);
		// 设置角色列表 （ID）
		List<Integer> roleIds = sysRoleService.findRolesByUserId(sysUser.getUserId()).stream().map(SysRole::getRoleId)
				.collect(Collectors.toList());
		userInfo.setRoles(ArrayUtil.toArray(roleIds, Integer.class));

		// 设置权限列表（menu.permission）
		Set<String> permissions = new HashSet<>();
		roleIds.forEach(roleId -> {
			List<String> permissionList = sysMenuService.findMenuByRoleId(roleId).stream()
					.filter(menuVo -> StringUtils.isNotEmpty(menuVo.getPermission())).map(MenuVO::getPermission)
					.collect(Collectors.toList());
			permissions.addAll(permissionList);
		});
		userInfo.setPermissions(ArrayUtil.toArray(permissions, String.class));
		return userInfo;
	}

	/**
	 * 分页查询用户信息（含有角色信息）
	 *
	 * @param page    分页对象
	 * @param userDTO 参数列表
	 * @return
	 */
	@Override
	public IPage getUsersWithRolePage(Page page, UserDTO userDTO) {
		return baseMapper.getUserVosPage(page, userDTO, new DataScope());
	}

	/**
	 * 分页查询用户信息（含有角色信息）
	 *
	 * @param page    分页对象
	 * @param userDTO 参数列表
	 * @return
	 */
	@Override
	public IPage getUsersWithRolePageNoLock(Page page, UserDTO userDTO) {
		return baseMapper.getUserVosPageNoLock(page, userDTO, new DataScope());
	}

	@Override
	public IPage selectUserList(Page page, UserDTO userDTO) {
		if (SecurityUtils.getRoles().contains(1) || SecurityUtils.getRoles().contains(2)) {
			return baseMapper.getUserVosPage(page, userDTO, new DataScope());
		}
		Integer deptId = userDTO.getDeptId();
//		if (null != userDTO.getDeptId()) {
//			deptId = userDTO.getDeptId();
//		}
		userDTO.setDeptId(SecurityUtils.getUser().getDeptId());
//		if(SecurityUtils.getRoles().contains(7)) {
//			List Listrole = new ArrayList();
//			Listrole.add(7);
//			userDTO.setRole(Listrole);
//		}
		List<UserVO> userList = sysUserMapper.selectByDeptId(userDTO);
		SysDeptRelation sysDeptRelation = new SysDeptRelation();
		sysDeptRelation.setAncestor(userDTO.getDeptId());
		List<SysDeptRelation> sysDeptRelationList = sysDeptRelationMapper.selectListByAncestor(sysDeptRelation);
		List<Integer> ids1 = sysDeptRelationList.stream().map(p -> p.getDescendant()).collect(Collectors.toList());
		List<UserVO> userList1 = new ArrayList<>();
		List<UserVO> userList2 = new ArrayList<>();
		for (Integer id : ids1) {
			userDTO.setDeptId(id);
			List<UserVO> userList3 = sysUserMapper.selectByDeptId(userDTO);
			userList1.addAll(userList3);
		}
		List<Integer> ids2 = userList1.stream().map(p -> p.getUserId()).collect(Collectors.toList());
		for (int i = 0; i < userList1.size(); i++) {
			if (userList1.get(i).getRoleList().size() != 0) {
				if (userList1.get(i).getRoleList().get(0).getRoleId().equals(4)) {
					userDTO.setUserId(ids2.get(i));
					UserVO userVO = sysUserMapper.selectByUserId(userDTO);
					userList2.add(userVO);
				}
			}
		}
		userList.addAll(userList2);
		List<Integer> ids = userList.stream().map(p -> p.getUserId()).collect(Collectors.toList());
		for (int i = 0; i < userList.size(); i++) {
			List<SysRole> sysRoleList1 = sysRoleService.findRolesByUserId(ids.get(i));
			userList.get(i).setRoleList(sysRoleList1);
		}

		if (null != deptId) {
			userList = userList.stream().filter(item -> deptId.equals(item.getDeptId())).collect(Collectors.toList());
			;
		}


		//每页数据条数
		Page<UserVO> page1 = new Page<>();
		int current = new Long(page.getCurrent()).intValue();
		int size = new Long(page.getSize()).intValue();

		int count = userList.size();
		List<UserVO> pageList = new ArrayList<>();
		//计算当前页第一条数据的下标
		int currId = current > 1 ? (current - 1) * size : 0;
		for (int i = 0; i < size && i < count - currId; i++) {
			pageList.add(userList.get(currId + i));
		}
		page1.setSize(size);
		page1.setCurrent(current);
		page1.setTotal(count);
		//计算分页总页数
		page1.setPages(count % 10 == 0 ? count / 10 : count / 10 + 1);
		page1.setRecords(pageList);
		return page1;
	}

	/**
	 * 通过ID查询用户信息
	 *
	 * @param id 用户ID
	 * @return 用户信息
	 */
	@Override
	public UserVO selectUserVoById(Integer id) {
		return baseMapper.getUserVoById(id);
	}

	/**
	 * 删除用户
	 *
	 * @param sysUser 用户
	 * @return Boolean
	 */
	@Override
	@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#sysUser.username")
	public Boolean deleteUserById(SysUser sysUser) {
		sysUserRoleService.deleteByUserId(sysUser.getUserId());
		this.removeById(sysUser.getUserId());
		return Boolean.TRUE;
	}

	@Override
	@Caching(evict = {@CacheEvict(value = CacheConstants.CLIENT_DETAILS_KEY, key = "#userDto.username"),
			@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")})
	public R<Boolean> updateUserInfo(UserDTO userDto) {
		UserVO userVO = baseMapper.getUserVoById(userDto.getUserId());
		SysUser sysUser = new SysUser();
		String newPwd = ENCODER.encode(userDto.getNewpassword1());
		if (StrUtil.isNotBlank(userDto.getPassword()) && StrUtil.isNotBlank(userDto.getNewpassword1())) {
			if (ENCODER.matches(userDto.getPassword(), userVO.getPassword())) {
				sysUser.setPassword(newPwd);
			} else {
				log.info("原密码错误，修改密码失败:{}", userDto.getUsername());
				return R.failed("原密码错误，修改失败");
			}
		}
		sysUser.setPhone(userDto.getPhone());
		sysUser.setUserId(userVO.getUserId());
		sysUser.setAvatar(userDto.getAvatar());
		if (null != userDto.getNewpassword1()) {
			sysUser.setW1(Md5Utils.getMD5(userDto.getNewpassword1().getBytes()));
		}


		List<SysUser> list = this.getUserByPlatId(userVO.getPlatId());

		if (this.updateById(sysUser)) {
			for (SysUser sysUser1 : list) {
				if (!sysUser.getUserId().equals(sysUser1.getUserId())) {
					sysUser1.setPassword(newPwd);
					this.updateById(sysUser1);
				}
			}
		}

		return R.ok();
	}

//	@Override
//	@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
//	public R<Boolean> updateUserInfo(UserDTO userDto) {
////		UserVO userVO = baseMapper.getUserVoByUsername(userDto.getUsername());
//		/**
//		 * 20210102
//		 */
//		UserVO userVO = baseMapper.getUserVoById(userDto.getUserId());
//		SysUser sysUser = new SysUser();
//		if (StrUtil.isNotBlank(userDto.getPassword()) && StrUtil.isNotBlank(userDto.getNewpassword1())) {
//			if (ENCODER.matches(userDto.getPassword(), userVO.getPassword())) {
//				sysUser.setPassword(ENCODER.encode(userDto.getNewpassword1()));
//			} else {
//				log.info("原密码错误，修改密码失败:{}", userDto.getUsername());
//				return R.failed("原密码错误，修改失败");
//			}
//		}
//		sysUser.setPhone(userDto.getPhone());
//		sysUser.setUserId(userVO.getUserId());
//		sysUser.setAvatar(userDto.getAvatar());
//		if (null != userDto.getNewpassword1()) {
//			sysUser.setW1(Md5Utils.getMD5(userDto.getNewpassword1().getBytes()));
//		}
//
//
//		if (null != userDto.getDeptId()) {
//			SysDept sysDept = sysDeptService.getById(userDto.getDeptId());
//			if (null != sysDept) {
//				sysUser.setDrugEntId(sysDept.getLevel1Id() + "");
//			}
//		}
//		return R.ok(this.updateById(sysUser));
//	}


	@Override
	@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
	public R<Boolean> updateUserInfoVo(UserVO userVO) {
		SysUser sysUser = super.getById(userVO.getUserId());
		if (null != sysUser) {
			sysUser.setWxOpenid(userVO.getWxOpenid());
			sysUser.setUserId(userVO.getUserId());
			sysUser.setAvatar(userVO.getAvatar());
			return R.ok(this.updateById(sysUser));
		}

		return R.ok();
	}

	@Override
	@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
	public Boolean updateUser(UserDTO userDto) {
		SysUser sysUser = new SysUser();
		BeanUtils.copyProperties(userDto, sysUser);
		sysUser.setUpdateTime(LocalDateTime.now());
		if (null != userDto.getPassword()) {
			sysUser.setW1(Md5Utils.getMD5(userDto.getPassword().getBytes()));
		}
		if (StrUtil.isNotBlank(userDto.getPassword())) {
			sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
		}

		this.updateById(sysUser);

		sysUserRoleService
				.remove(Wrappers.<SysUserRole>update().lambda().eq(SysUserRole::getUserId, userDto.getUserId()));
		userDto.getRole().forEach(roleId -> {
			SysUserRole userRole = new SysUserRole();
			userRole.setUserId(sysUser.getUserId());
			userRole.setRoleId(roleId);
			userRole.insert();
		});
		return Boolean.TRUE;
	}

	/**
	 * 查询上级部门的用户信息
	 *
	 * @param username 用户名
	 * @return R
	 */
	@Override
	public List<SysUser> listAncestorUsers(String username) {
		SysUser sysUser = this.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));

		SysDept sysDept = sysDeptService.getById(sysUser.getDeptId());
		if (sysDept == null) {
			return null;
		}

		Integer parentId = sysDept.getParentId();
		return this.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getDeptId, parentId));
	}

	/**
	 * 获取当前用户的子部门信息
	 *
	 * @return 子部门列表
	 */
	private List<Integer> getChildDepts() {
		Integer deptId = SecurityUtils.getUser().getDeptId();
		// 获取当前部门的子部门
		return sysDeptRelationService
				.list(Wrappers.<SysDeptRelation>query().lambda().eq(SysDeptRelation::getAncestor, deptId)).stream()
				.map(SysDeptRelation::getDescendant).collect(Collectors.toList());
	}


	public List<SysUser> getUserByPlatId(String platId) {
		SysUser sysUser = new SysUser();
		sysUser.setDelFlag("0");
		sysUser.setPlatId(platId);
		return super.list(Wrappers.<SysUser>query(sysUser));
	}

	@Override
	public List<UserVO> getUserVoByPlatId(UserDTO userDTO) {
		List<Integer> list = new ArrayList<>();
		list.add(5);
		list.add(6);
		list.add(10);
		// 患者教育-HCP角色
		list.add(13);
		userDTO.setRole(list);
		return baseMapper.getUserVoByPlatId(userDTO);
	}

	@Override
	public R updatePwd(UserPwd userPwd) {
		String p1 = userPwd.getP1();
		String p2 = userPwd.getP2();
		String userName = userPwd.getUserName();
		String code = userPwd.getCode();

//		String key = CacheConstants.DEFAULT_CODE_KEY + LoginTypeEnum.SMS.getType() + StringPool.AT + userName;
//
//		redisTemplate.setKeySerializer(new StringRedisSerializer());
//
//		if (!redisTemplate.hasKey(key)) {
//			throw new ValidateCodeException("请输入正确验证码");
//		}
//
//		Object codeObj = redisTemplate.opsForValue().get(key);
//
//		if (codeObj == null) {
//			throw new ValidateCodeException("请输入正确验证码");
//		}
//
//		String saveCode = codeObj.toString();
//		if (StrUtil.isBlank(saveCode)) {
//			redisTemplate.delete(key);
//			throw new ValidateCodeException("请输入正确验证码");
//		}
//
//		if (!StrUtil.equals(saveCode, code)) {
//			redisTemplate.delete(key);
//			throw new ValidateCodeException("请输入正确验证码");
//		}
//
		List<SysUser> userList = baseMapper.selectList(Wrappers.<SysUser>lambdaQuery().
				eq(SysUser::getUsername, userName));
		if (CollectionUtils.isEmpty(userList)) {
			return R.failed("用户不存在");
		}

		if (!p1.equals(p2)) {
			return R.failed("两次密码输入不一致");
		}
		for (SysUser sysUser : userList) {
			sysUser.setUserId(sysUser.getUserId());
			sysUser.setW1(Md5Utils.getMD5(p1.getBytes()));
			sysUser.setPassword(ENCODER.encode(p1));
			baseMapper.updateById(sysUser);
		}

//		redisTemplate.delete(key);
		return R.ok();
	}

	@Override
	public SysUser getByIdIgnore(String id) {
		return baseMapper.getByIdIgnore(id);
	}
}
